#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <netdb.h>
#include <sys/time.h>
#include <string.h>
#include <errno.h>

/* global variables */
struct hostent *hi;
int sockfd;
int articleCount=0;

/* debugging options. no need to change it */
#ifdef DEBUG
#define DUDEL
#endif
#ifdef DEBUG2 
#define DUDEL
#endif
#ifdef DUDEL
FILE *debugfp;
#endif

/* get next number in string */
char *number(char *sp, int *intPtr)
{
int i,c;

 if (sp==NULL) { *intPtr=0; return(sp); }
 for (i=0;sp[i]!=' ';i++) if (sp[i]=='\0') break;
 c=sp[i];
 sp[i]='\0';
 sscanf(sp,"%d",intPtr);
 sp[i++]=c;
 return sp+i;
}

/* exit on failure, but close connection first */
void failure()
{
 close(sockfd);
 exit(-1);
}


void connect_to_nntphost(char *host)
{
char *ptr;
struct in_addr *aptr;
struct in_addr saddr;
struct sockaddr_in	address;

/*
	Find the internet address of the NNTP server
*/
 hi = gethostbyname(host);
 if ( hi == NULL )
 {
  fprintf(stderr,"%s :",host);
  perror("Could not get host information:");
  exit(-1);
 }
 fprintf(stderr,"Official host name: %s\n", hi->h_name );
 while ( (ptr = *(hi->h_aliases)) != NULL )
 {
  fprintf(stderr,"Alias %s\n", ptr );
  hi->h_aliases++;
 }
 if ( hi->h_addrtype != AF_INET )
 {
  fprintf(stderr,"Unsupported address type.\n" );
  exit(-1);
 }
 while ( (aptr = (struct in_addr *)*(hi->h_addr_list)++) != NULL )
 {
  saddr = *aptr;
  fprintf(stderr,"Address: %s\n", inet_ntoa(*aptr) );
 }

 /*
  Create a socket
 */
 sockfd = socket( AF_INET, SOCK_STREAM, 0 );
 if ( sockfd == -1 )
 {
  perror("Socket failed:");
  exit(-1);
 }
 
 address.sin_family = AF_INET;
 address.sin_port = 119;  /* NNTP port */
 address.sin_addr = saddr;

/* 
 Establish a connection
*/
 if ( connect( sockfd, (struct sockaddr *)&address, sizeof address ) == -1 )
 {
  perror("Connect failed:");
  failure();
 }
 fprintf(stderr,"Connected to %s\n",hi->h_name);
}

void get_message_index()
{
int count,low,high,response,lastread;
char *sp,buf[1024],group[512];
FILE *indexfp,*ifp,*tmpfp;

 ifp = fopen( "sucknewsrc", "r" );
 if ( ifp == NULL )
 {
  perror( "sucknewsrc:" );
  failure();
 }

 tmpfp = fopen( "suck.tmp", "w" );
 if ( tmpfp == NULL )
 {
  perror("suck.tmp: ");
  failure();
 }

 indexfp = fopen( "suck.index", "w" );
 if (indexfp==NULL)
 {
  perror("suck.index");
  failure();
 }

 while ( fscanf( ifp, "%s %d\n", group, &lastread ) == 2 )
 {
  sprintf(buf,"group %s\r\n",group);
#ifdef DEBUG2
  fprintf(debugfp,"group %s\n",group);
#endif
  sputline(sockfd,buf);
  sgetline(sockfd,buf,1024);
  sp = number(buf, &response);
  if (sp == NULL)
  {
   fprintf(stderr,"Protocol error!\n");
   failure();
  }
  if ( response != 211 )
  {
   fprintf(stderr,"GROUP <%s>, unexpected response %d\n", group, response );
   continue; /* next newsgroup */
  }
  sp = number( sp, &count );
  sp = number( sp, &low );
  sp = number( sp, &high );
  
  fprintf(tmpfp, "%s %d\n", group, high);
  
  if ( lastread && lastread < high )
  {
   if ( lastread < low ) lastread = low;
   sprintf( buf, "xhdr Message-ID %d-\r\n", lastread+1 );
   fprintf(stderr,"%s...articles %d-%d\n", group, lastread+1, high);
   sputline( sockfd, buf );
   sgetline( sockfd, buf, 1024 );
   number(buf, &response);
   if (response == 221)
   {
    do
    {
     sgetline( sockfd, buf, 1024 );
     if (buf[0] != '.' )
     {
      for ( sp = buf; *sp != '<'; sp++ ) ;
      fprintf( indexfp, "%s\n", sp );
      articleCount++;
     }
    } while ( buf[0] != '.' || buf[1] != '\0' );
   } /* end if response */
   else
   {
    fprintf(stderr,"Unexpected response %d getting IDs\n", response );
    failure();
   }
  } /* end if lastread */
 } /* end while */
 fclose(indexfp);
 fclose(tmpfp);
 fclose(ifp);
}


void get_articles()
{
int response,len;
char buf[1024];
FILE *ifp;

 ifp = fopen( "suck.sorted", "r" );
 if ( ifp == NULL )
 {
  perror("suck.sorted: ");
  failure();
 }
 
 sprintf(buf, "article ");
 while ( fgets(&buf[8], 512, ifp) != NULL )
 {
  fprintf(stderr,"%d\n", articleCount--);
  len=strlen(buf);
  buf[len++]='\r';
  buf[len++]='\n';
  buf[len]='\0';

#ifdef DUDEL
fputs("SENDING REQUEST\n",debugfp);
fputs(buf,debugfp);
#endif
  sputline(sockfd,buf);
  sgetline(sockfd,buf,1024);
  number(buf, &response);

  if ( response != 220 )
  {
   fprintf(stderr,"*** Warning: Unexpected response\n");
   fputs(buf,stderr);
   fputc('\n',stderr);
  }

  do
  {
   len=sgetline(sockfd, buf, 1024);
   puts(buf);
  } while ( (len!=1) || (buf[0] != '.') );
 
   sprintf( buf, "article " );
 } /* end while */
 fclose(ifp);
}


main(int argc, char **argv)
{
int	selret, recvret, readmask, i, response;
char *sp, *ptr, buf[1024];

#ifdef DUDEL
debugfp=fopen("suck.debug","w");
#endif

 connect_to_nntphost( (argc>1) ? argv[1] : getenv("NNTPSERVER") );

 /*
  Get the announcement line
 */

 sgetline(sockfd, buf, 1024);
 fprintf(stderr,"%s\n", buf );


 sputline(sockfd,"mode reader\r\n");
 sgetline(sockfd,buf,1024); /* Again the announcement */


 fprintf(stderr,"%s\n",buf);

 get_message_index();
 
 if (articleCount>0)
 {
  fprintf(stderr,"Removing duplicates from list of %d articles.\n",
													articleCount);
  unlink("suck.sorted");
  system("sort suck.index | uniq > suck.sorted");
  get_articles();
  fprintf(stderr,"skipped %d duplicates\n",articleCount);
 }
 else fprintf(stderr,"No articles.\n");

 fprintf(stderr,"closing connection to %s\n",hi->h_name);
 sputline( sockfd, "quit\r\n" );
 close(sockfd);
#ifdef DUDEL
fclose(debugfp);
#endif
}

 
/*
 get a CR\NL terminated line from newsserver.
 sgetline doesn't really care for maxlen yet ;)
*/
int	sgetline(int fd, char *sbuf, int maxlen)
{
int	ret;
char *ptr;
static int count=0;
static char tempbuf[1024];

 if (count>0)			/* some bytes left from previous sgetline */
 {
  ptr=strstr(tempbuf,"\r\n");
#ifdef DEBUG2
fprintf(debugfp,"debug count %d\n",count);
fputs(tempbuf,debugfp);
fputc('\n',debugfp);
#endif
  if (ptr!=NULL) goto returnline;
 }

 do
 {
  ret=recv(fd, &tempbuf[count], 1024-count, 0);
  if (ret<1) { perror("socket error: "); failure(); }
  count+=ret;
  tempbuf[count]='\0'; /* for strstr to work */
  ptr=strstr(tempbuf,"\r\n");
 } while (ptr==NULL);

returnline:
 ret=ptr-tempbuf;			/* somehow it's the strlen */
 bcopy(tempbuf,sbuf,ret);   /* copy the whole line */
 ptr=sbuf+ret;				/* point to \r\n in sbuf */
 *ptr='\0';					/* terminate sbuf, forget \r\n */
 count= count - ret - 2;	/* count=remaining bytes in tempbuf */
 if (count>0) 
 {
  bcopy(tempbuf+ret+2,tempbuf,count); /* bcopy cares for overlap */
  tempbuf[count]='\0';		/* terminate tempbuf */
 }
 else { count=0; *tempbuf='\0'; }
#ifdef DEBUG
fprintf(debugfp,"RETURNING ",count);
fputs(sbuf,debugfp);
fputc('\n',debugfp);
#endif
 return(ret);				/* return strlen of sbuf */
}


int sputline( int fd, char *outbuf )
{
#ifdef DEBUG2
fprintf(debugfp,"\ndebug putline:");
fputs(buf,debugfp);
#endif
 send( fd, outbuf, strlen(outbuf), 0 );
}
